#include "DS50FragmentGenerator.hh"
#include "fhiclcpp/ParameterSet.h"
#include <boost/lexical_cast.hpp>
#include "cetlib/exception.h"
#include "messagefacility/MessageLogger/MessageLogger.h"
#include <unistd.h>

ds50::DS50FragmentGenerator::DS50FragmentGenerator(const fhicl::ParameterSet &ps): 
  artdaq::CommandableFragmentGenerator(ps)
{
  sleep_us_ = ps.get<int> ("sleep_us", 0);
  init_only_ = ps.get<bool> ("init_only", false);
}

void ds50::DS50FragmentGenerator::start () { 

  perfreset ();

  start_ (); 
}


void ds50::DS50FragmentGenerator::pause () { 
  pause_ ();
}

void ds50::DS50FragmentGenerator::resume () { 
  resume_ ();
} 

void ds50::DS50FragmentGenerator::stop () { 
  stop_ ();
}

std::string ds50::DS50FragmentGenerator::report () {
  return "run: " + boost::lexical_cast<std::string>(stats_.run_number) + 
         "\ncall_count: " + boost::lexical_cast<std::string>(stats_.call_count) +
         "\navg_frag_size: " + boost::lexical_cast<std::string>(stats_.avg_frag_size) + 
         "\nread_count: " + boost::lexical_cast<std::string>(stats_.read_count) +
         "\n" + report_ ();
}

void ds50::DS50FragmentGenerator::perfreset () {
  
  stats_.run_number = run_number();
  stats_.read_count = stats_.call_count = 0;
  stats_.avg_frag_size = 0;
  perfreset_ ();
}


bool ds50::DS50FragmentGenerator::getNext_ (artdaq::FragmentPtrs & output) { 
  if (init_only_) stop ();

  if (sleep_us_ > 0) usleep (sleep_us_);

  bool r = false;

  stats_.read_count -= output.size ();

  r = getNext__(output); 

  stats_.avg_frag_size = (stats_.avg_frag_size * stats_.call_count + output.size ()) / (stats_.call_count + 1);
  stats_.call_count ++;
  stats_.read_count += output.size ();

  if (!r) mf::LogDebug("getNext_") << "stopped after " << stats_.read_count << " events";
  return r;
}

